#! /usr/bin/perl
#
# Copyright (c) Daniel Hottinger <hodaniel@student.ethz.ch>
# Released under the terms of the GNU General Public License v2 or later
# 
# Created: Wed Apr 20 18:20:07 CEST 2005
# Last modified: Wed Apr 20 21:01:48 CEST 2005
# Changes:
#  tgr - new rx_in/tx_in fields

use strict;
use warnings;

sub decode_nbo_32($) {
	return unpack("N", pack("L", hex(shift)));
}

sub decode_nbo_64($) {
	my $hex = shift;
	my ($a, $b) = (hex(substr($hex, 0, 8)), hex(substr($hex, 8, 8)));
	($a, $b) = unpack("NN", pack("LL", $a, $b));
	return $a + ($b << 32);
}

my %data = ();
my $lastname = '';
my @values = qw(rx_rate tx_rate rx_total tx_total time);

while (<>) {
	chomp;
	s/#.*$//;
	next if /^\s*$/;


	my ($name, $parent, $handle, $parent_handle, $attribute, $rx_rate,
	    $tx_rate, $rx_total, $tx_total, $rx_in, $tx_in,
	    $time) = split /\s+/;
	my ($ts_secs, $ts_usecs) = split /:/, $time;

	if (! defined $data{$name}) {
		$data{$name} = {};
	}

	$data{$name}->{$attribute} = {
		'rx_rate'  => decode_nbo_32($rx_rate),
		'tx_rate'  => decode_nbo_32($tx_rate),
		'rx_total' => decode_nbo_64($rx_total),
		'tx_total' => decode_nbo_64($tx_total),
		'rx_in'    => decode_nbo_64($rx_in),
		'tx_in'    => decode_nbo_64($tx_in),
		'time'     => scalar localtime(decode_nbo_64($ts_secs) + 
			     (decode_nbo_64($ts_usecs) / 1000000)),
	};
	$data{$name}->{'parent'} = $parent;
	$data{$name}->{'handle'} = $handle;

	if (($name ne $lastname) and ($lastname ne '')) {
		printf "%s <%s,%s>\n           ", $lastname, 
			$data{$lastname}->{'parent'},
			$data{$lastname}->{'handle'};

		delete $data{$lastname}->{'parent'};
		delete $data{$lastname}->{'handle'};

		my @attributes = sort keys %{$data{$lastname}};
		for (@values) {
			printf "%10s", $_;
		}
		print "\n";

		for $attribute (@attributes) {
			printf "%11s", $attribute;
			for my $value (@values) {
				my $fmt = $value eq 'time' ? '      %10s' : '%10s';
				printf $fmt, $data{$lastname}->{$attribute}->{$value};
			}
			print "\n";
		}
		print "\n\n\n\n";
	}
	$lastname = $name;
}
