%{
 #include <bmon/bmon.h>
 #include <bmon/db.h>
 #include <bmon/utils.h>
%}

%locations
%pure_parser
%token-table
%error-verbose
%name-prefix="db_filter_"

%union {
	unsigned int i;
	char *s;
	struct db_token *t;
}

%{
 extern struct db_token * db_filter_out;
 extern int db_filter_lex(YYSTYPE *, YYLTYPE *);
 extern void db_filter_error(char *);
%}

%token <s> NAME
%token <i> NUMBER WILDCARD SEPARATOR ATTR_SEP

%type <t> tok filter selection

%start input

%%
input:
	/* empty */
		{ db_filter_out = NULL; }
	| filter
		{ db_filter_out = $1; }
			
	;

filter:
	selection
		{ $$ = $1; }
	| selection ATTR_SEP NAME
		{
			struct db_token *p, *t = xcalloc(1, sizeof(*t));
			t->t_flags |= DB_T_ATTR;
			t->t_name = $3;

			for (p = $1; p->t_next; p = p->t_next);
			p->t_next = t;
			$$ = $1;
		}
	;

selection:
	tok
		{ $$ = $1; }
	| tok SEPARATOR selection
		{
			$$ = $1;
			$$->t_next = $3;
		}
	;

tok:
	WILDCARD
		{
			$$ = xcalloc(1, sizeof(struct db_token));
			$$->t_flags |= DB_T_WILDCARD;
		}
	| NUMBER
		{
			$$ = xcalloc(1, sizeof(struct db_token));
			$$->t_id = $1;
		}
	| NAME
		{
			$$ = xcalloc(1, sizeof(struct db_token));
			$$->t_name = $1;
		}
	;

%%
 void db_filter_error(char *s)
 {
 	quit("parse error: %s\n", s);
 }
