/*
 * group.c		Group Management
 *
 * Copyright (c) 2001-2005 Thomas Graf <tgraf@suug.ch>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <bmon/bmon.h>
#include <bmon/node.h>
#include <bmon/group.h>
#include <bmon/utils.h>

static struct group_data *group_defs;

struct group_data *lookup_group_data(const char *name)
{
	struct group_data *gd;

	for (gd = group_defs; gd; gd = gd->gd_next)
		if (!strcmp(gd->gd_name, name))
		    return gd;

	return NULL;
}

void add_group_def(const char *name, const char *title,
		   const char *left_maj_title,
		   const char *left_min_title,
		   const char *right_maj_title,
		   const char *right_min_title)
{
	struct group_data *gd;

	if (lookup_group_data(name))
		return;
	
	gd = xcalloc(1, sizeof(*gd));

	gd->gd_name = strdup(name);
	gd->gd_title = strdup(title);
	gd->gd_left_maj_title = strdup(left_maj_title);
	gd->gd_left_min_title = strdup(left_min_title);
	gd->gd_right_maj_title = strdup(right_maj_title);
	gd->gd_right_min_title = strdup(right_min_title);

	gd->gd_next = group_defs;
	group_defs = gd;


	DBG(1, "[DBG] New group definition \"%s\"\n", name);
}

void add_group_derived(const char *name, const char *title,
		       const char *template)
{
	struct group_data *gdt, *gd;

	if (lookup_group_data(name))
		return;

	gdt = lookup_group_data(template);
	if (gdt == NULL) {
		xwarn("Cannot find group definition for group \"%s\"\n", template);
		return;
	}

	gd = xcalloc(1, sizeof(*gd));

	gd->gd_name = strdup(name);
	gd->gd_title = strdup(title);
	gd->gd_left_maj_title = strdup(gdt->gd_left_maj_title);
	gd->gd_left_min_title = strdup(gdt->gd_left_min_title);
	gd->gd_right_maj_title = strdup(gdt->gd_right_maj_title);
	gd->gd_right_min_title = strdup(gdt->gd_right_min_title);

	gd->gd_next = group_defs;
	group_defs = gd;

	DBG(1, "[DBG] New derived group definition \"%s\" from (%s)\n",
	    name, template);
}


static void __init group_init(void)
{
	add_group_def("intf", "Interfaces",
		      "RX Rate", "#",
		      "TX Rate", "#");
}
